# Generated by Django 4.1.2 on 2022-11-10 18:27

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('book_manager', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='bookauthor',
            options={'ordering': ('book', 'order'), 'verbose_name': 'book author', 'verbose_name_plural': 'book authors'},
        ),
    ]



from typing import List

from django.urls import path, URLPattern

from .views import (
    ProjectListView
)


app_name: str = "sphinx_hosting"

urlpatterns: List[URLPattern] = [
    path('', ProjectListView.as_view(), name='project--list'),

]



import datetime
import logging
import csv
from typing import Dict, Any

from django.contrib.auth import get_user_model
from nameparser import HumanName

from .models import Book, BookAuthor, Author, Binding, Publisher, Reading, Shelf

logger = logging.getLogger('book_manager.importers')

User = get_user_model()


class GoodreadsImporter:
    """
    Import data into our database from a Goodreeads CSV Export.

    A Goodreads export has these columns:

    +---------------------------------+----------------+------------------------------+
    | Column name                     | Type           | Notes                        |
    +=================================+================+==============================+
    | Book Id                         | int, unique    | goodreads internal id        |
    +---------------------------------+----------------+------------------------------+
    | Title                           | str            |                              |
    +---------------------------------+----------------+------------------------------+
    | Author                          | str            | First Last                   |
    +---------------------------------+----------------+------------------------------+
    | Author l-f                      | str            | Last, First                  |
    +---------------------------------+----------------+------------------------------+
    | Additional Authors              | str            | First Last1, First Last2...  |
    +---------------------------------+----------------+------------------------------+
    | ISBN                            | str            | value is "=" if empty        |
    +---------------------------------+----------------+------------------------------+
    | ISBN13                          | str            | value is "=" if empty        |
    +---------------------------------+----------------+------------------------------+
    | My Rating                       | int            |                              |
    +---------------------------------+----------------+------------------------------+
    | Average Rating                  | float          | 2 decimals                   |
    +---------------------------------+----------------+------------------------------+
    | Publisher                       | str            | can be empty                 |
    +---------------------------------+----------------+------------------------------+
    | Binding                         | str            | can be empty                 |
    +---------------------------------+----------------+------------------------------+
    | Number of Pages                 | int            | can be empty                 |
    +---------------------------------+----------------+------------------------------+
    | Year Published                  | int            | can be empty                 |
    +---------------------------------+----------------+------------------------------+
    | Original Publication Year       | int            | can be empty                 |
    +---------------------------------+----------------+------------------------------+
    | Date read                       | date           | YYYY/MM/DD                   |
    +---------------------------------+----------------+------------------------------+
    | Date added                      | date           | YYYY/MM/DD                   |
    +---------------------------------+----------------+------------------------------+
    | Bookshelves                     | str            | comma separated              |
    +---------------------------------+----------------+------------------------------+
    | Bookshelves with positions      | str            | NAME (#NUM), comma sep       |
    +---------------------------------+----------------+------------------------------+
    | Exclusive Shelf                 | str            | NAME                         |
    +---------------------------------+----------------+------------------------------+
    | My Review                       | text           | can be empty                 |
    +---------------------------------+----------------+------------------------------+
    | Spoiler                         | text           | can be empty                 |
    +---------------------------------+----------------+------------------------------+
    | Private Notes                   | text           | can be empty                 |
    +---------------------------------+----------------+------------------------------+
    | Read count                      | int            |                              |
    +---------------------------------+----------------+------------------------------+
    | Owned copies                    | int            |                              |
    +---------------------------------+----------------+------------------------------+

    """

    def __init__(self) -> None:
        self.binding_map: Dict[str, Binding] = {}
        self.publisher_map: Dict[str, Publisher] = {}
        self.authors_map: Dict[str, Author] = {}

    def load_lookups(self, filename: str) -> None:
        """
        Find the unique bindings, publishers and authors in the Goodreads export
        CSV ``filename`` and create them in the database as necessary.

        Args:
            filename: the filename of the CSV file to read
        """
        bindings = set()
        publishers = set()
        authors = set()
        with open(filename, encoding='utf-8') as csvfile:
            reader = csv.DictReader(csvfile)
            for row in reader:
                if row['Binding']:
                    bindings.add(row['Binding'])
                if row['Publisher']:
                    publishers.add(row['Publisher'])
                authors.add(row['Author'])
                if row['Additional Authors']:
                    other = row['Additional Authors'].split(', ')
                    authors.update(set(other))

        for binding in list(bindings):
            self.binding_map[binding], created = Binding.objects.get_or_create(name=binding)
            if created:
                logger.info('%s.binding.created name="%s"', self.__class__.__name__, binding)
        for publisher in list(publishers):
            self.publisher_map[publisher], created = Publisher.objects.get_or_create(name=publisher)
            if created:
                logger.info('%s.publisher.created name="%s"', self.__class__.__name__, publisher)
        for name in list(authors):
            n = HumanName(name)
            full_name = str(n)
            author, created = Author.objects.get_or_create(full_name=full_name)
            if created:
                logger.info('%s.author.created full_name="%s"', self.__class__.__name__, full_name)
            author.first_name = n.first
            author.middle_name = n.middle
            author.last_name = n.last
            author.save()
            self.authors_map[full_name] = author

    def import_book(self, row: Dict[str, Any], overwrite: bool = False) -> Book:
        """
        Get or create a :py:class:`Book` based on ``row``, a row from our
        :py:class:`csv.DictReader` reader of our Goodreads export.

        Args:
            row: a row from our Goodreads export

        Keyword Args:
            overwrite: if ``True``, overwrite any existing book data for this book

        Returns:
            A :py:class:`Book` instance
        """
        book, created = Book.objects.get_or_create(title=row['Title'])
        if created or (not created and overwrite):
            isbn = row['ISBN'][1:].strip('"')
            isbn13 = row['ISBN13'][1:].strip('"')
            original_publication_year = row['Original Publication Year']
            if not original_publication_year:
                original_publication_year = None
            book.isbn = isbn if isbn else None
            book.isbn13 = isbn13 if isbn13 else None
            book.num_pages = row['Number of Pages'] if row['Number of Pages'] else None
            book.year_published = row['Year Published'] if row['Year Published'] else None
            book.original_publication_year = original_publication_year
            if row['Binding']:
                book.binding = self.binding_map[row['Binding']]
            if row['Publisher']:
                book.publisher = self.publisher_map[row['Publisher']]
            book.save()
            primary_author = HumanName(row['Author'])
            book.authors.clear()
            author_order = 1
            BookAuthor.objects.create(book=book, author=self.authors_map[str(primary_author)], order=author_order)
            book.authors.add(self.authors_map[str(primary_author)])
            if row['Additional Authors']:
                others = row['Additional Authors'].split(', ')
                for author in others:
                    author_order += 1
                    n = HumanName(author)
                    BookAuthor.objects.create(book=book, author=self.authors_map[str(n)], order=author_order)
        if created:
            logger.info('%s.book.created title="%s"', self.__class__.__name__, book.title)
        else:
            logger.info('%s.book.updated title="%s"', self.__class__.__name__, book.title)
        return book

    def import_reading(self, book: Book, user: User, row: Dict[str, Any]) -> None:
        """
        Import the data for the :py:class:`Reading` record for ``user``.

        Args:
            book: the book for which we're importing reading data
            user: the user whose reading data we're importing
            row: the row from the Goodreads CSV, as output by :py:class:`csv.DictReader`
        """
        shelf, _ = Shelf.objects.get_or_create(reader=user, name=row['Exclusive Shelf'])
        created = False
        try:
            reading = Reading.objects.get(book=book, reader=user)
        except Reading.DoesNotExist:
            created = True
            reading = Reading(book=book, reader=user, shelf=shelf)
        reading.date_added = datetime.datetime.strptime(row['Date Added'], '%Y/%m/%d').date()
        if row['Date Read']:
            reading.date_read = datetime.datetime.strptime(row['Date Read'], '%Y/%m/%d').date()
        if row['Private Notes']:
            reading.private_notes = row['Private Notes']
        if row['My Review']:
            reading.review = row['My Review']
        reading.read_count = row['Read Count'] if row['Read Count'] else 0
        reading.rating = row['My Rating']
        reading.save()
        if created:
            logger.info('%s.reading.created user=%s title="%s"', self.__class__.__name__, user.username, book.title)
        else:
            logger.info('%s.reading.updated user=%s title="%s"', self.__class__.__name__, user.username, book.title)

    def run(self, filename: str, user: User, overwrite: bool = False) -> None:
        """
        Load the books in the CSV identified by ``filename`` into
        the database, splitting each row into appropriate :py:class:`Book`,
        :py:class:`Author`, :py:class:`Publisher` and :py:class:`Binding`
        records, creating the foreign keys and many-to-many targets as
        needed.

        :py:class:`Reading` data will always be overwritten, and
        :py:class:`Book` data will be preserved, unless ``override`` is
        ``True``.

        Args:
            filename: the filename of the Goodreads CSV export file

        Keyword Args:
            overwrite: if ``True``, overwrite any existing :py:class:`Book` with data from the CSV
        """
        self.load_lookups(filename)
        with open(filename, encoding='utf-8') as csvfile:
            reader = csv.DictReader(csvfile)
            for row in reader:
                book = self.import_book(row, overwrite=overwrite)
                self.import_reading(book, user, row)



from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class BookManagerAppConfig(AppConfig):
    name: str = "book_manager"
    label: str = "book_manager"
    verbose_name: str = _("Book Manager")
    default_auto_field: str = "django.db.models.AutoField"



# Generated by Django 4.1.2 on 2022-11-10 18:03

import book_manager.validators
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django_extensions.db.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Author',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('first_name', models.CharField(max_length=255, verbose_name='First name')),
                ('last_name', models.CharField(max_length=255, verbose_name='Last name')),
                ('middle_name', models.CharField(blank=True, default=None, max_length=255, null=True, verbose_name='Middle name')),
                ('full_name', models.CharField(max_length=255, unique=True, verbose_name='Full name')),
            ],
            options={
                'verbose_name': 'author',
                'verbose_name_plural': 'authors',
            },
        ),
        migrations.CreateModel(
            name='Binding',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Binding type', max_length=128, verbose_name='Binding type')),
            ],
            options={
                'verbose_name': 'binding',
                'verbose_name_plural': 'bindings',
            },
        ),
        migrations.CreateModel(
            name='Book',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('title', models.CharField(help_text='The title of the book', max_length=255, unique=True, verbose_name='Book Title')),
                ('slug', django_extensions.db.fields.AutoSlugField(blank=True, editable=False, help_text='Used in the URL for the book. Must be unique.', populate_from='title', unique=True, verbose_name='Slug')),
                ('isbn', models.CharField(blank=True, default=None, max_length=16, null=True, verbose_name='ISBN')),
                ('isbn13', models.CharField(blank=True, default=None, max_length=16, null=True, verbose_name='ISBN')),
                ('num_pages', models.PositiveIntegerField(blank=True, default=None, null=True, verbose_name='Num Pages')),
                ('year_published', models.IntegerField(blank=True, default=None, null=True, verbose_name='Year Published')),
                ('original_publication_year', models.IntegerField(blank=True, default=None, null=True, verbose_name='Original Publication Year')),
            ],
            options={
                'verbose_name': 'book',
                'verbose_name_plural': 'books',
            },
        ),
        migrations.CreateModel(
            name='Publisher',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('name', models.CharField(help_text='Publisher name', max_length=255, verbose_name='Publisher name')),
            ],
            options={
                'verbose_name': 'publisher',
                'verbose_name_plural': 'publishers',
            },
        ),
        migrations.CreateModel(
            name='Shelf',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(help_text='Name of a shelf on which books can live', max_length=128, verbose_name='Shelf name')),
                ('reader', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='shelves', to=settings.AUTH_USER_MODEL, verbose_name='Reader')),
            ],
            options={
                'verbose_name': 'shelf',
                'verbose_name_plural': 'shelves',
            },
        ),
        migrations.CreateModel(
            name='Reading',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', django_extensions.db.fields.CreationDateTimeField(auto_now_add=True, verbose_name='created')),
                ('modified', django_extensions.db.fields.ModificationDateTimeField(auto_now=True, verbose_name='modified')),
                ('rating', models.PositiveIntegerField(default=0, validators=[django.core.validators.MaxValueValidator(5), django.core.validators.MinValueValidator(0)], verbose_name='Rating')),
                ('private_notes', models.TextField(blank=True, default=None, help_text='Private notes that only you can see', null=True, validators=[book_manager.validators.NoHTMLValidator()], verbose_name='Private Notes')),
                ('review', models.TextField(blank=True, default=None, help_text='Notes that anyone can see', null=True, validators=[book_manager.validators.NoHTMLValidator()], verbose_name='Review')),
                ('read_count', models.PositiveIntegerField(help_text="How many times you've read this book", verbose_name='Read count')),
                ('date_added', models.DateField(help_text='Date this book was added to your reading list', verbose_name='Date added')),
                ('date_read', models.DateField(default=None, help_text='Date you first read this book', null=True, verbose_name='Date read')),
                ('book', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='readings', to='book_manager.book')),
                ('reader', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='readings', to=settings.AUTH_USER_MODEL, verbose_name='Reader')),
                ('shelf', models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='readings', to='book_manager.shelf')),
            ],
            options={
                'verbose_name': 'reading',
                'verbose_name_plural': 'readings',
            },
        ),
        migrations.CreateModel(
            name='BookAuthor',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(default=1, verbose_name='Author order')),
                ('author', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='book_manager.author', verbose_name='Author')),
                ('book', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='book_manager.book')),
            ],
            options={
                'verbose_name': 'book author',
                'verbose_name_plural': 'book authors',
                'unique_together': {('book', 'author', 'order')},
            },
        ),
        migrations.AddField(
            model_name='book',
            name='authors',
            field=models.ManyToManyField(related_name='books', through='book_manager.BookAuthor', to='book_manager.author', verbose_name='Authors'),
        ),
        migrations.AddField(
            model_name='book',
            name='binding',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='books', to='book_manager.binding'),
        ),
        migrations.AddField(
            model_name='book',
            name='publisher',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='books', to='book_manager.publisher'),
        ),
        migrations.AddField(
            model_name='book',
            name='readers',
            field=models.ManyToManyField(related_name='books', through='book_manager.Reading', to=settings.AUTH_USER_MODEL, verbose_name='Readers'),
        ),
    ]



from django.contrib.auth import get_user_model
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django.utils.translation import gettext_lazy as _

from django_extensions.db.models import TimeStampedModel
from django_extensions.db.fields import AutoSlugField

from .validators import NoHTMLValidator


F = models.Field
M2M = models.ManyToManyField
FK = models.ForeignKey


class Binding(models.Model):
    """
    A binding of a :py:class:`Book` ("ebook", "mass market paperback",
    "hardcover", etc.).  Books have zero or one bindings.
    """

    name: F = models.CharField(
        _('Binding type'),
        max_length=128,
        help_text=_('Binding type')
    )

    class Meta:
        verbose_name: str = _('binding')
        verbose_name_plural: str = _('bindings')


class Publisher(TimeStampedModel, models.Model):
    """
    A publisher of a :py:class:`Book`.  Books have zero or one publishers.
    """

    name: F = models.CharField(
        _('Publisher name'),
        max_length=255,
        help_text=_('Publisher name')
    )

    class Meta:
        verbose_name: str = _('publisher')
        verbose_name_plural: str = _('publishers')


class Author(TimeStampedModel, models.Model):
    """
    An author of a :py:class:`Book`.  Books can have multiple authors.
    """

    first_name: F = models.CharField(
        _('First name'),
        max_length=255,
    )
    last_name: F = models.CharField(
        _('Last name'),
        max_length=255,
    )
    middle_name: F = models.CharField(
        _('Middle name'),
        max_length=255,
        null=True,
        blank=True,
        default=None
    )
    full_name: F = models.CharField(
        _('Full name'),
        max_length=255,
        unique=True
    )

    class Meta:
        verbose_name: str = _('author')
        verbose_name_plural: str = _('authors')


class Book(TimeStampedModel, models.Model):

    title: F = models.CharField(
        _('Book Title'),
        help_text=_('The title of the book'),
        max_length=255,
        unique=True
    )
    slug: F = AutoSlugField(
        _('Slug'),
        unique=True,
        populate_from='title',
        help_text=_('Used in the URL for the book. Must be unique.')
    )
    isbn: F = models.CharField(
        _('ISBN'),
        max_length=16,
        null=True,
        blank=True,
        default=None,
    )
    isbn13: F = models.CharField(
        _('ISBN'),
        max_length=16,
        null=True,
        blank=True,
        default=None,
    )
    num_pages: F = models.PositiveIntegerField(
        _('Num Pages'),
        null=True,
        blank=True,
        default=None,
    )
    year_published: F = models.IntegerField(
        _('Year Published'),
        null=True,
        blank=True,
        default=None,
    )
    original_publication_year: F = models.IntegerField(
        _('Original Publication Year'),
        null=True,
        blank=True,
        default=None,
    )
    binding: FK = models.ForeignKey(
        Binding,
        on_delete=models.SET_NULL,
        related_name='books',
        null=True
    )
    publisher: FK = models.ForeignKey(
        Publisher,
        on_delete=models.CASCADE,
        related_name='books',
        null=True
    )
    authors: M2M = models.ManyToManyField(
        Author,
        verbose_name=_('Authors'),
        related_name='books',
        through='BookAuthor'
    )
    readers: M2M = models.ManyToManyField(
        get_user_model(),
        verbose_name=_('Readers'),
        related_name='books',
        through='Reading'
    )

    @property
    def primary_author(self) -> Author:
        """
        Return the top-billed author for this book.  This is the author
        with ``order=1`` in our :py:class:`BookAuthor` through table.

        Returns:
            The :py:class:`Author` object for the primary author
        """
        return self.authors.get(bookauthor__order=1)

    @property
    def other_authors(self) -> "models.QuerySet[Author]":
        """
        Return all authors other than the top-billed author for this book.
        These are the authors with ``order>1`` in our :py:class:`BookAuthor`
        through table.

        Returns:
            The queryset of :py:class:`Author` objects for the non-primary
            author.
        """
        return self.authors.filter(bookauthor__order__gt=1)

    class Meta:
        verbose_name: str = _('book')
        verbose_name_plural: str = _('books')


class BookAuthor(models.Model):
    """
    This is a through table between :py:class:`Book` and :py:class:`Author` that
    allows us to keep our book authors in the correct order.
    """

    book = models.ForeignKey(
        Book,
        on_delete=models.CASCADE,
    )
    author = models.ForeignKey(
        Author,
        on_delete=models.CASCADE,
        verbose_name=_('Author'),
    )
    order = models.PositiveIntegerField(
        _('Author order'),
        default=1
    )

    class Meta:
        unique_together = ('book', 'author', 'order')
        verbose_name: str = _('book author')
        verbose_name_plural: str = _('book authors')
        ordering = ('book', 'order',)


class Shelf(models.Model):
    """
    This model is used to organize :py:class:`Reading` instances for a user into
    buckets ("read", "to-read", "abandoned").  Shelves are per-user.
    """

    name: F = models.CharField(
        _('Shelf name'),
        max_length=128,
        help_text=_('Name of a shelf on which books can live')
    )

    reader = models.ForeignKey(
        get_user_model(),
        on_delete=models.CASCADE,
        verbose_name=_('Reader'),
        related_name='shelves'
    )

    class Meta:
        verbose_name: str = _('shelf')
        verbose_name_plural: str = _('shelves')


class Reading(TimeStampedModel, models.Model):
    """
    This model holds user-specific data about a reading of a :py:class:`Book`
    """

    rating = models.PositiveIntegerField(
        _('Rating'),
        default=0,
        validators=[MaxValueValidator(5), MinValueValidator(0)]
    )
    private_notes = models.TextField(
        _('Private Notes'),
        help_text=_('Private notes that only you can see'),
        blank=True,
        null=True,
        default=None,
        validators=[NoHTMLValidator()]
    )
    review = models.TextField(
        _('Review'),
        help_text=_('Notes that anyone can see'),
        blank=True,
        null=True,
        default=None,
        validators=[NoHTMLValidator()]
    )
    read_count = models.PositiveIntegerField(
        _('Read count'),
        help_text=_("How many times you've read this book")
    )
    date_added = models.DateField(
        _('Date added'),
        help_text=_("Date this book was added to your reading list")
    )
    date_read = models.DateField(
        _('Date read'),
        help_text=_("Date you first read this book"),
        null=True,
        default=None
    )

    reader = models.ForeignKey(
        get_user_model(),
        on_delete=models.CASCADE,
        verbose_name=_('Reader'),
        related_name='readings'
    )
    book = models.ForeignKey(
        Book,
        on_delete=models.CASCADE,
        related_name='readings'
    )
    shelf = models.ForeignKey(
        Shelf,
        on_delete=models.SET_NULL,
        null=True,
        default=None,
        related_name='readings'
    )

    class Meta:
        verbose_name: str = _('reading')
        verbose_name_plural: str = _('readings')



from setuptools import setup, find_packages

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name="django-book-manager",
    version="0.2.0",
    packages=find_packages(),
    include_package_data=True,
    package_data={'book_manager': ["py.typed"]},
    install_requires=[
        "django-extensions",
        "nameparser"
    ],
    author="Caltech IMSS ADS",
    author_email="cmalek@caltech.edu",
    url="https://github.com/caltechads/django-book-manager",
    description="Reusable Django app for managing lists of books with ratings.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=['django'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Testing',
    ],
)



from typing import Any

from django.core.exceptions import ValidationError
from django.utils.deconstruct import deconstructible
from django.utils.html import strip_tags


@deconstructible
class NoHTMLValidator:
    """
    Verify that field contains no HTML
    """
    message: str = 'Cannot contain any HTML'
    code: str = 'invalid'

    def __call__(self, value: str) -> None:
        if not value == strip_tags(value):
            raise ValidationError(self.message, code=self.code)

    def __eq__(self, other: Any) -> bool:
        return (
            isinstance(other, NoHTMLValidator)
        )
